#ifndef __ALL_IMFS__
#define __ALL_IMFS__
#include "IMF/FP32/acos_s_ep.cl"
#include "IMF/FP32/acos_s_ha.cl"
#include "IMF/FP32/acos_s_la.cl"
#include "IMF/FP32/acosh_s_ep.cl"
#include "IMF/FP32/acosh_s_ha.cl"
//#include "IMF/FP32/acosh_s_la.cl"
//#include "IMF/FP32/acospi_s_la.cl"
#include "IMF/FP32/asin_s_ep.cl"
#include "IMF/FP32/asin_s_ha.cl"
//#include "IMF/FP32/asin_s_la.cl"
#include "IMF/FP32/asinh_s_ep.cl"
#include "IMF/FP32/asinh_s_ha.cl"
//#include "IMF/FP32/asinh_s_la.cl"
//#include "IMF/FP32/asinpi_s_la.cl"
#include "IMF/FP32/atan2_s_ep.cl"
#include "IMF/FP32/atan2_s_ha.cl"
#include "IMF/FP32/atan2_s_la.cl"
#include "IMF/FP32/atan_s_ep.cl"
#include "IMF/FP32/atan_s_ep_noLUT.cl"
#include "IMF/FP32/atan_s_ha.cl"
#include "IMF/FP32/atan_s_la.cl"
#include "IMF/FP32/atanh_s_ep.cl"
#include "IMF/FP32/atanh_s_ha.cl"
//#include "IMF/FP32/atanh_s_la.cl"
//#include "IMF/FP32/atanpi_s_la.cl"
//#include "IMF/FP32/cbrt_s_la.cl"
//#include "IMF/FP32/cbrt_s_la_noLUT.cl"
#include "IMF/FP32/cos_s_ep.cl"
#include "IMF/FP32/cos_s_ha.cl"
//#include "IMF/FP32/cos_s_la.cl"
//#include "IMF/FP32/cos_s_noLUT.cl"
#include "IMF/FP32/cosh_s_ep.cl"
#include "IMF/FP32/cosh_s_ha.cl"
//#include "IMF/FP32/cosh_s_la.cl"
//#include "IMF/FP32/cospi_s_la.cl"
//#include "IMF/FP32/cospi_s_noLUT.cl"
#include "IMF/FP32/erf_s_ep.cl"
#include "IMF/FP32/erf_s_ep_noLUT.cl"
#include "IMF/FP32/erf_s_ha.cl"
//#include "IMF/FP32/erf_s_la.cl"
//#include "IMF/FP32/erf_s_la_noLUT.cl"
#include "IMF/FP32/erfc_s_ha.cl"
//#include "IMF/FP32/erfc_s_la.cl"
//#include "IMF/FP32/erfc_s_la_noLUT.cl"
#include "IMF/FP32/exp10_s_ha.cl"
//#include "IMF/FP32/exp10_s_la.cl"
#include "IMF/FP32/exp2_s_ha.cl"
#include "IMF/FP32/exp2_s_la.cl"
#include "IMF/FP32/exp_s_ha.cl"
#include "IMF/FP32/exp_s_la.cl"
#include "IMF/FP32/expm1_s_ep.cl"
#include "IMF/FP32/expm1_s_ha.cl"
//#include "IMF/FP32/expm1_s_la.cl"
#include "IMF/FP32/hypot_s_ha.cl"
#include "IMF/FP32/hypot_s_la.cl"
#include "IMF/FP32/lgamma_s_ep.cl"
//#include "IMF/FP32/lgamma_s_noFP64.cl"
#include "IMF/FP32/lgamma_s_noLUT.cl"
#include "IMF/FP32/ln_s_ep.cl"
#include "IMF/FP32/ln_s_ep_noLUT.cl"
#include "IMF/FP32/ln_s_ha.cl"
//#include "IMF/FP32/ln_s_la.cl"
#include "IMF/FP32/log10_s_ep.cl"
#include "IMF/FP32/log10_s_ha.cl"
#include "IMF/FP32/log10_s_ha_noLUT.cl"
//#include "IMF/FP32/log10_s_la.cl"
#include "IMF/FP32/log1p_s_ep.cl"
#include "IMF/FP32/log1p_s_ha.cl"
//#include "IMF/FP32/log1p_s_la.cl"
#include "IMF/FP32/log2_s_ep.cl"
#include "IMF/FP32/log2_s_ha.cl"
#include "IMF/FP32/log2_s_ha_noLUT.cl"
//#include "IMF/FP32/log2_s_la.cl"
#include "IMF/FP32/pow_s_ep.cl"
#include "IMF/FP32/pow_s_ha.cl"
//#include "IMF/FP32/pow_s_la.cl"
//#include "IMF/FP32/pow_s_la_noLUT.cl"
//#include "IMF/FP32/pow_s_prev.cl"
//#include "IMF/FP32/pown_s_la.cl"
//#include "IMF/FP32/pown_s_la_noLUT.cl"
//#include "IMF/FP32/pown_s_prev.cl"
//#include "IMF/FP32/powr_s_la.cl"
//#include "IMF/FP32/powr_s_la_noLUT.cl"
//#include "IMF/FP32/rootn_s_la.cl"
#include "IMF/FP32/sin_s_ep.cl"
#include "IMF/FP32/sin_s_ha.cl"
//#include "IMF/FP32/sin_s_la.cl"
//#include "IMF/FP32/sin_s_noLUT.cl"
#include "IMF/FP32/sincos_s_dp.cl"
#include "IMF/FP32/sincos_s_ep_noLUT.cl"
#include "IMF/FP32/sincos_s_ha.cl"
//#include "IMF/FP32/sincos_s_la.cl"
//#include "IMF/FP32/sincos_s_la_noLUT.cl"
//#include "IMF/FP32/sincos_s_noLUT.cl"
#include "IMF/FP32/sincospi_s_ep.cl"
//#include "IMF/FP32/sincospi_s_la.cl"
#include "IMF/FP32/sinh_s_ep.cl"
#include "IMF/FP32/sinh_s_ha.cl"
//#include "IMF/FP32/sinh_s_la.cl"
//#include "IMF/FP32/sinpi_s_la.cl"
//#include "IMF/FP32/sinpi_s_noLUT.cl"
#include "IMF/FP32/tan_s_ha.cl"
#include "IMF/FP32/tan_s_ha_noLUT.cl"
//#include "IMF/FP32/tan_s_la.cl"
#include "IMF/FP32/tanh_s_ep.cl"
#include "IMF/FP32/tanh_s_ep_noLUT.cl"
#include "IMF/FP32/tanh_s_ha.cl"
#include "IMF/FP32/tanh_s_la.cl"
//#include "IMF/FP32/tanh_s_la_noLUT.cl"
//#include "IMF/FP32/tanpi_s_la.cl"
#include "IMF/FP32/tgamma_s_ep.cl"
//#include "IMF/FP32/tgamma_s_noFP64.cl"
#include "IMF/FP64/acos_d_ep.cl"
#include "IMF/FP64/acos_d_ha.cl"
//#include "IMF/FP64/acos_d_la.cl"
#include "IMF/FP64/acosh_d_ha.cl"
//#include "IMF/FP64/acosh_d_la.cl"
//#include "IMF/FP64/acospi_d_la.cl"
#include "IMF/FP64/asin_d_ep.cl"
#include "IMF/FP64/asin_d_ha.cl"
//#include "IMF/FP64/asin_d_la.cl"
#include "IMF/FP64/asinh_d_ha.cl"
//#include "IMF/FP64/asinh_d_la.cl"
//#include "IMF/FP64/asinpi_d_la.cl"
#include "IMF/FP64/atan2_d_ep.cl"
#include "IMF/FP64/atan2_d_ha.cl"
//#include "IMF/FP64/atan2_d_la.cl"
#include "IMF/FP64/atan_d_ep.cl"
#include "IMF/FP64/atan_d_ep_noLUT.cl"
#include "IMF/FP64/atan_d_ha.cl"
//#include "IMF/FP64/atan_d_la.cl"
//#include "IMF/FP64/atan_d_la_noLUT.cl"
#include "IMF/FP64/atanh_d_ha.cl"
//#include "IMF/FP64/atanh_d_la.cl"
//#include "IMF/FP64/atanpi_d_la.cl"
//#include "IMF/FP64/cbrt_d_la.cl"
//#include "IMF/FP64/cbrt_d_la_noLUT.cl"
#include "IMF/FP64/cos_d_ep.cl"
#include "IMF/FP64/cos_d_ha.cl"
#include "IMF/FP64/cos_d_ha_noLUT.cl"
//#include "IMF/FP64/cos_d_la.cl"
#include "IMF/FP64/cosh_d_ha.cl"
//#include "IMF/FP64/cosh_d_la.cl"
//#include "IMF/FP64/cospi_d_la.cl"
#include "IMF/FP64/erf_d_ep.cl"
#include "IMF/FP64/erf_d_ep_noLUT.cl"
#include "IMF/FP64/erf_d_ha.cl"
//#include "IMF/FP64/erf_d_la.cl"
//#include "IMF/FP64/erf_d_la_noLUT.cl"
#include "IMF/FP64/erfc_d_ha.cl"
//#include "IMF/FP64/erfc_d_la.cl"
//#include "IMF/FP64/erfc_d_la_noLUT.cl"
#include "IMF/FP64/exp10_d_ep.cl"
#include "IMF/FP64/exp10_d_ha.cl"
//#include "IMF/FP64/exp10_d_la.cl"
#include "IMF/FP64/exp2_d_ep.cl"
#include "IMF/FP64/exp2_d_ha.cl"
//#include "IMF/FP64/exp2_d_la.cl"
//#include "IMF/FP64/exp2_d_la_noLUT.cl"
#include "IMF/FP64/exp_d_ep.cl"
#include "IMF/FP64/exp_d_ep_noLUT.cl"
#include "IMF/FP64/exp_d_ha.cl"
//#include "IMF/FP64/exp_d_la.cl"
//#include "IMF/FP64/exp_d_la_noLUT.cl"
#include "IMF/FP64/expm1_d_ep.cl"
#include "IMF/FP64/expm1_d_ha.cl"
//#include "IMF/FP64/expm1_d_la.cl"
#include "IMF/FP64/hypot_d_ha.cl"
#include "IMF/FP64/hypot_d_la.cl"
#include "IMF/FP64/ln_d_ep.cl"
#include "IMF/FP64/ln_d_ep_noLUT.cl"
#include "IMF/FP64/ln_d_ha.cl"
//#include "IMF/FP64/ln_d_la.cl"
//#include "IMF/FP64/ln_d_la_noLUT.cl"
#include "IMF/FP64/log10_d_ha.cl"
//#include "IMF/FP64/log10_d_la.cl"
//#include "IMF/FP64/log10_d_la_noLUT.cl"
#include "IMF/FP64/log1p_d_ha.cl"
//#include "IMF/FP64/log1p_d_la.cl"
//#include "IMF/FP64/log1p_d_la_noLUT.cl"
#include "IMF/FP64/log2_d_ha.cl"
//#include "IMF/FP64/log2_d_la.cl"
//#include "IMF/FP64/log2_d_la_noLUT.cl"
#include "IMF/FP64/pow_d_ha.cl"
//#include "IMF/FP64/pow_d_la.cl"
//#include "IMF/FP64/pow_d_la_noLUT.cl"
//#include "IMF/FP64/pown_d_la.cl"
//#include "IMF/FP64/pown_d_la_noLUT.cl"
//#include "IMF/FP64/powr_d_la.cl"
//#include "IMF/FP64/powr_d_la_noLUT.cl"
//#include "IMF/FP64/rootn_d_la.cl"
#include "IMF/FP64/sin_d_ep.cl"
#include "IMF/FP64/sin_d_ha.cl"
#include "IMF/FP64/sin_d_ha_noLUT.cl"
//#include "IMF/FP64/sin_d_la.cl"
#include "IMF/FP64/sincos_d_ep.cl"
#include "IMF/FP64/sincos_d_ep_noLUT.cl"
#include "IMF/FP64/sincos_d_ha.cl"
//#include "IMF/FP64/sincos_d_la.cl"
//#include "IMF/FP64/sincos_d_la_noLUT.cl"
#include "IMF/FP64/sincospi_d_ep.cl"
//#include "IMF/FP64/sincospi_d_la.cl"
#include "IMF/FP64/sinh_d_ha.cl"
//#include "IMF/FP64/sinh_d_la.cl"
//#include "IMF/FP64/sinpi_d_la.cl"
#include "IMF/FP64/tan_d_ep.cl"
#include "IMF/FP64/tan_d_ha.cl"
//#include "IMF/FP64/tan_d_la.cl"
#include "IMF/FP64/tanh_d_ha.cl"
//#include "IMF/FP64/tanh_d_la.cl"
//#include "IMF/FP64/tanh_d_la_noLUT.cl"
//#include "IMF/FP64/tanpi_d_la.cl"
#endif
