/*========================== begin_copyright_notice ============================

Copyright (C) 2024 Intel Corporation

SPDX-License-Identifier: MIT

============================= end_copyright_notice ===========================*/

    UNKNOWN,
    VERTEX_SHADER,
    HULL_SHADER,
    DOMAIN_SHADER,
    GEOMETRY_SHADER,
    TASK_SHADER,
    MESH_SHADER,
    PIXEL_SHADER,
    COMPUTE_SHADER,
    OPENCL_SHADER,
    RAYTRACING_SHADER,
    END,
