; NOTE: Assertions have been autogenerated by utils/update_test_checks.py UTC_ARGS: --version 5
;=========================== begin_copyright_notice ============================
;
; Copyright (C) 2024 Intel Corporation
;
; SPDX-License-Identifier: MIT
;
;============================ end_copyright_notice =============================

; RUN: igc_opt -debugify --igc-gas-resolve --opaque-pointers -S < %s | FileCheck %s
; REQUIRES: llvm-14-plus, opaque-ptr-fix

%st = type {ptr addrspace(4)}

define ptr addrspace(4) @should_add_addrspacecast_with_metadata(ptr addrspace(1) %src) {
  %1 = inttoptr i32 13 to ptr addrspace(2)
  %2 = addrspacecast ptr addrspace(2) %1 to ptr addrspace(4)
  %3 = getelementptr %st, ptr addrspace(1) %src, i32 0, i32 0
  %pointer_value = load ptr addrspace(4), ptr addrspace(1) %3
  %pointer_value2 = addrspacecast ptr addrspace(4) %pointer_value to ptr addrspace(2)
  store i32 13, ptr addrspace(4) %pointer_value
  store i32 14, ptr addrspace(2) %pointer_value2
  ret ptr addrspace(4) %pointer_value
}

!igc.functions = !{!0}
!0 = !{ptr addrspace(4) (ptr addrspace(1))* @should_add_addrspacecast_with_metadata, !1}
!1 = !{!2}
!2 = !{!"function_type", i32 0}
